import { parseSafeFloat } from '../parseFloats.js';
import { ValueType } from './ValueType.js';
export class ValueTypeRegistry {
    constructor() {
        this.valueTypeNameToValueType = {};
        this.register(new ValueType('string', () => '', (value) => value, (value) => value));
        this.register(new ValueType('boolean', () => false, (value) => typeof value === 'string' ? value.toLowerCase() === 'true' : value, (value) => value));
        this.register(new ValueType('float', () => 0, (value) => typeof value === 'string' ? parseSafeFloat(value, 0) : value, (value) => value));
        this.register(new ValueType('integer', () => 0n, (value) => BigInt(value), (value) => Number.MIN_SAFE_INTEGER <= value && value <= Number.MAX_SAFE_INTEGER
            ? Number(value)
            : value.toString()));
        this.register(new ValueType('id', () => '', (value) => value, (value) => value));
    }
    register(valueType) {
        if (valueType.name in this.valueTypeNameToValueType) {
            throw new Error(`already registered value type ${valueType.name}`);
        }
        this.valueTypeNameToValueType[valueType.name] = valueType;
    }
    get(valueTypeName) {
        if (!(valueTypeName in this.valueTypeNameToValueType)) {
            throw new Error(`can not find value type with name '${valueTypeName}`);
        }
        return this.valueTypeNameToValueType[valueTypeName];
    }
    getAllNames() {
        return Object.keys(this.valueTypeNameToValueType);
    }
}
//# sourceMappingURL=ValueTypeRegistry.js.map